#!/usr/local/bin/perl -w
#
#	webc.cgi
#
#	This is a simple wrapper program for webc to run files
#	through webc on the fly.
#
#	Because of security issues, I need to put webc into a
#	restricted mode, one which doesn't allow localcode to
#	retrieve just any file.  The only files allowed are in
#		/usr/local/lib/webc/cgi
#
#	Only process files ending in "*.wc" as a simple attempt
#	to keep the program from displaying any file specified.
#
#	Dave Regan
#	regan@peak.org
#	http://cornvalley.peak.org/
#	Copyright 1998 Dave Regan, all rights reserved
#

###
###	Configuration
###
use strict;
use vars qw(%ENV $InCGI $NoLocalcode %Vars $Version
		$Webc $WebcInclude $WebcPath);

#	Tell us where to find webc at runtime
$Webc = "/usr/local/bin/webc";

#	Tell us what to use as a WEBCPATH
$WebcPath	= "/usr/local/lib/webc/lc/cgi";
$WebcInclude	= "/usr/local/lib/webc/include";

###
###	Global variables
###

$Version	= 'webc.cgi v0.01  regan@ao.com'; 	# Version ID

# Webc control
$InCGI = 1;             # Put webc into CGI mode
$NoLocalcode = 2;       # Allow #localcode, but only from WEBCPATH.  No ../.
require $Webc;		# Get the compiler

###
###	Main program
###

    my($fname, $home, $user);

    $| = 1;
    print "Content-type: text/html\n\n";
#   for $home (sort keys %ENV) { print "$home: $ENV{$home}<br>\n"; }
    $ENV{'WEBCINCLUDE'} = $WebcInclude;
    $ENV{'WEBCPATH'} = $WebcPath;
    ParseQueryString();

    # Find the file specified.  This should be absolute or ~user based.
    $fname = $Vars{'file'} ||
    	Error("Usage: http://$ENV{'SERVER_NAME'}$ENV{'SCRIPT_NAME'}?" .
    		"file=filename.wc");
    if ($fname =~ m#^~([^/]*)(.*)#)
    	{
    	($user, $fname) = ($1, $2);
    	$home = (getpwnam($user))[7] || Error("User $user not found");
    	$fname = "$home$fname";
    	}

    # Ensure it is *.wc
    Error("File $fname doesn't end in <i>.wc</i>") if ($fname !~ /\.wc$/);

    # Compile it
    Compile($fname);


###
###	Error
###
###	Spit out a simple error message.
###
sub Error
    {
    my($msg) = @_;

    print << "EOF";
    	<html>
    	<head>
    	<title>webc.cgi error</title>
    	</head>
    	<body>
    	$msg
    	</body>
    	</html>
EOF
    exit 0;
    }


###
###	ParseQueryString
###
###	Break apart a query string.
###	This probably will need to be smarter.
###	It should unescape values.
###
sub ParseQueryString
    {
    my($field, @fields, $name, $value);

    return if (!defined($ENV{'QUERY_STRING'}) || $ENV{'QUERY_STRING'} eq "");
    @fields = split(/\&/, $ENV{'QUERY_STRING'});
    for $field (@fields)
    	{
    	($name, $value) = split(/=/, $field, 2);
    	$Vars{$name} = $value if (defined($value));
    	}
    }
